@@ -4,7 +4,7 @@
 
 extern int _valid_stm(SEXP x);
 
-// ceeboo 2010/8
+// ceeboo 2010/8+10
 //
 // sum (collapse) the rows of x into the column groups 
 // defined in index.
@@ -15,13 +15,11 @@ SEXP _row_tsums(SEXP x, SEXP R_index, SEXP R_na_rm, SEXP R_verbose) {
     if (!inherits(R_index, "factor"))
 	error("'index' not of class 'factor'");
 
-    int *p, *q, k, n, m, l;
-    SEXP _v, _i, _j, __v, r, s;
+    int *p, *q, k, n, m, f, l;
+    SEXP _v, _i, _j, __i, __v, r, s;
 
     if (LENGTH(R_index) != INTEGER(VECTOR_ELT(x, 4))[0])
 	error("'index' invalid length");
-    //m = ;
-    m = INTEGER(VECTOR_ELT(x, 3))[0];
 
 #ifdef _TIME_H
     // code section times
@@ -32,45 +30,65 @@ SEXP _row_tsums(SEXP x, SEXP R_index, SEXP R_na_rm, SEXP R_verbose) {
     p = INTEGER(PROTECT(allocVector(INTSXP, LENGTH(_i))));
     q = INTEGER(PROTECT(allocVector(INTSXP, LENGTH(_i))));
 
+    // sort by row indexes
+    for (int i = 0; i < LENGTH(_i); i++) {
+	p[i] = INTEGER(_i)[i];
+	q[i] = i;
+    }
+    if (LENGTH(_i))
+	R_qsort_int_I(p, q, 1, LENGTH(_i));
+
+    // sort row blocks by column indexes
+    //
+    // NOTE we change the sign with each block
+    //	    to ensure a change in key.
+    //
     _j = VECTOR_ELT(x, 1);
 
-    // NOTE tapply ignores NA(s) in argument INDEX.
-    //      So do we with a warning.
-    //      
-    // FIXME bailout if k * m > 2147483647.
+    f = 0;
     l = 0;
+    n = 0;
+    m = 0;
     for (int i = 0; i < LENGTH(_i); i++) {
-	k = INTEGER(R_index)[INTEGER(_j)[i] - 1];
+	k = INTEGER(R_index)[INTEGER(_j)[q[i]] - 1];
 	if (k == NA_INTEGER)
 	    continue;
-	k = INTEGER(_i)[i] - 1 + (k - 1) * m;
-	p[l] = k;
-	q[l] = i;
+	if (n != p[i]) {
+	    n  = p[i];
+	    if (f < l)
+		R_qsort_int_I(p, q, f, l);
+	    f = l + 1;
+	    m = (m) ? 0 : 1;
+	}
+	p[l] = (m) ? k : -k;
+	q[l] = q[i];
 	l++;
     }
-    if (l < LENGTH(_i))
-	warning("NA(s) in 'index'");
-    else
-	for (int i = 0; i < LENGTH(R_index); i++)
-	    if (INTEGER(R_index)[i] == NA_INTEGER) {
-		warning("NA(s) in 'index'");
-		break;
-	    }
+    if (l) {
+	R_qsort_int_I(p, q, f, l);
+	// FIXME this may be time-consuming.
+	if (l < LENGTH(_i))
+	    warning("NA(s) in 'index'");
+	else
+	    for (int i = 0; i < LENGTH(R_index); i++)
+		if (INTEGER(R_index)[i] == NA_INTEGER) {
+		    warning("NA(s) in 'index'");
+	    	    break;
+		}
+    }
 
-    if (l)
-	R_qsort_int_I(p, q, 1, l);
-
-    n =  0;
-    k = -1;
+    // count
+    n = 0;
+    k = 0;
     for (int i = 0; i < l; i++)
-	if (p[i] > k) {
-	    k = p[i];
+	if (k != p[i]) {
+	    k  = p[i];
 	    n++;
 	}
 
     // NOTE use REALSXP to avoid overflows.
     r = PROTECT(allocVector(VECSXP, 6));
-    SET_VECTOR_ELT(r, 0, ( _i = allocVector(INTSXP,  n)));
+    SET_VECTOR_ELT(r, 0, (__i = allocVector(INTSXP,  n)));
     SET_VECTOR_ELT(r, 1, ( _j = allocVector(INTSXP,  n)));
     SET_VECTOR_ELT(r, 2, (__v = allocVector(REALSXP, n)));
     SET_VECTOR_ELT(r, 3, VECTOR_ELT(x, 3));
@@ -100,44 +118,45 @@ SEXP _row_tsums(SEXP x, SEXP R_index, SEXP R_na_rm, SEXP R_verbose) {
 #ifdef _TIME_H
     t1 = clock();
 #endif
+    double *_z = REAL(__v);
     _v = VECTOR_ELT(x, 2);
 
-    n = -1;
-    k = -1;
+    n = 0;
+    k = 0;
     switch (TYPEOF(_v)) {
 	case LGLSXP:
 	case INTSXP:
 	    for (int i = 0; i < l; i++) {
-		if (p[i] > k) {
-		    k = p[i];
+		if (k != p[i]) {
+		    k  = p[i];
+		    INTEGER(__i)[n] = INTEGER(_i)[q[i]];
+		    INTEGER( _j)[n] = (k > 0) ? k : -k;
+		     _z = REAL(__v) + n;    
+		    *_z = 0;
 		    n++;
-		    INTEGER(_i)[n] =  k % m + 1;
-		    INTEGER(_j)[n] =  k / m + 1;
-		    REAL(__v)[n]   =  0;
 		}
 		int z = INTEGER(_v)[q[i]];
 		if (z != NA_INTEGER)
-		    REAL(__v)[n] += (double) z;
+		    *_z += (double) z;
 		else
 		    if (!LOGICAL(R_na_rm)[0])
-			REAL(__v)[n] += NA_REAL;
+			*_z += NA_REAL;
 	    }
 	    break;
 	case REALSXP:
 	    for (int i = 0; i < l; i++) {
-		if (p[i] > k) {
-		    k = p[i];
+		if (k != p[i]) {
+		    k  = p[i];
+		    INTEGER(__i)[n] = INTEGER(_i)[q[i]];
+		    INTEGER( _j)[n] = (k > 0) ? k : -k;
+		     _z = REAL(__v) + n;    
+		    *_z = 0;
 		    n++;
-		    // NOTE we expect that retrieving the i
-		    //      index value is less efficient
-		    INTEGER(_i)[n] =  k % m + 1;
-		    INTEGER(_j)[n] =  k / m + 1;
-		    REAL(__v)[n]   =  0;
 		}
 		double z = REAL(_v)[q[i]];
 		// NOTE with na.rm == TRUE the sum may be zero
 		if (!ISNAN(z) || !LOGICAL(R_na_rm)[0])
-		    REAL(__v)[n] += z;
+		    *_z += z;
 	    }
 	    break;
 	default:
